/**
 * Binary search tree.
 *
 * https://www.nada.kth.se/~snilsson/algoritmer/trad/
 *
 * @author Stefan Nilsson
 * @version 2017-02-08
 */
public class BinarySearchTree {
    private Node root;
    
    private static class Node {
        Node left, right;
        int data;
        
        public Node(int data) {
            this.data = data;
        }

        @Override
        public String toString() {
            return "[" + data + "]";
        }
    }

    /**
     * Creates an empty BinarySearchTree.
     */
    public BinarySearchTree() {
    }

    /**
     * Prints a picture of this tree to stdout.
     */
    public void print() {
        printTree(root, 0);
    }

    /**
     * Prints a picture of the tree (lying on its side)
     * indented by "padding" from the left margin.
     */
    private void printTree(Node root, int padding) {
        if (root == null) {
            return;
        }
        // Print right subtree.
        printTree(root.right, padding + 1);
        // Print the root:
        for (int i = padding; i > 0; i--) {
            System.out.print("    ");
        }
        System.out.println(root);
        // Print the left subtree.
        printTree(root.left, padding + 1);
    }
    
    /**
     * Returns true if the value is in this tree.
     */
     public boolean find(int value) {
        Node n = root;

        while (n != null) {
            if (n.data == value) {
                return true;
            }
            if (value < n.data) {
                n = n.left;
            } else {
                n = n.right;
            }
        }
        return false;
     }

    /**
     * Inserts the value (if it's not already there) in this tree.
     */
    public void insert(int value) {
        root = insert(value, root);
    }

    /**
     * Inserts a new node in the tree rooted at "root" and
     * returns the root of the modified tree.
     */
    private Node insert(int value, Node root) {
        if (root == null) {
            return new Node(value);
        }

        if (value == root.data) {
    	    return root; // do nothing, element already in tree
        }

        if (value < root.data) {
            root.left = insert(value, root.left);
        } else {
            root.right = insert(value, root.right);
        }
        return root;
    }     

    /**
     * Unit test.
     * Usage: java -ea BinarySearchTree
     */
    public static void main(String[] args) {
        BinarySearchTree t = new BinarySearchTree();
        /*
        t.root = new Node(5);
        t.root.left = new Node(3);
        t.root.right = new Node(8);
        t.root.right.right = new Node(10);
        */
        t.insert(5);
        t.insert(3);
        t.insert(8);
        t.insert(10);
        t.print();
        assert t.find(3);
        assert t.find(5);
        assert t.find(8);
        assert t.find(10);
        assert !t.find(2);
        assert !t.find(6);
        assert !t.find(12);
    }
}
